Imports System.Data
Imports System.Data.OleDb

Partial Class Optimization
    Inherits System.Web.UI.Page


#Region "Event Handler"

    Protected Sub Page_Load(ByVal sender As Object, _
    ByVal e As System.EventArgs) Handles Me.Load
        If Not Session("SelectedPID") Is Nothing Then
            Session("PageTitle") = "Investment Report Minimizing Risk"
        Else
            Response.Redirect("Portal.aspx")
        End If
        optimize()
    End Sub

    Protected Sub cmdOpt_Click(ByVal sender As Object, _
    ByVal e As System.EventArgs) Handles cmdOpt.Click
        optimize()
    End Sub

#End Region

#Region "Member Functions"

    Public Sub optimize()

        If txtCash.Text = "" Or txtMinRet.Text = "" Or txtMaxInv.Text = "" Then
            lblError.Text = "Please enter all input values!"
            Exit Sub
        End If

        'Step 1: Determine the worth of each stock in the selected PID 
        Dim connGetStockWorth As New OleDbConnection
        Dim cmdGetStockWorth As New OleDbCommand
        Dim daGetStockWorth As New OleDbDataAdapter
        Dim dsGetStockWorth As New DataSet
        connGetStockWorth.ConnectionString = ConfigurationManager.AppSettings("connString")
        cmdGetStockWorth.Connection = connGetStockWorth
        cmdGetStockWorth.CommandText = "exec qryOptimization '" & Session("SelectedPID") & "'"
        daGetStockWorth.SelectCommand = cmdGetStockWorth
        connGetStockWorth.Open()
        daGetStockWorth.Fill(dsGetStockWorth)
        connGetStockWorth.Close()

        'Step 2: Setup initial values and data structures
        Dim i, bestStock As Integer
        Dim cashUsed, currentReturn, finalReturn, finalVariance As Double

        Dim numStocks As Integer = dsGetStockWorth.Tables(0).Rows.Count
        'Stores worth of each stock; worth = avg price / std Dev of return
        'We set the stock with the highest worth as the bestStock
        Dim worth(numStocks - 1) As Double
        Dim maxWorth As Double = 0
        For i = 0 To numStocks - 1
            worth(i) = CDbl(dsGetStockWorth.Tables(0).Rows(i).Item(3))
            If (worth(i) > maxWorth) Then
                bestStock = i
                maxWorth = worth(i)
            End If
        Next
        'Stores the stock weights; initial value = 0; investment = cash * weight 
        Dim weights(numStocks - 1) As Double

        Dim CASH As Double = CDbl(txtCash.Text)
        Dim MINRET As Double = CDbl(txtMinRet.Text) / 100
        Dim MAXWT As Double = CDbl(txtMaxInv.Text) / 100
        lblVar.Text = "Infinity"
        lblReturn.Text = 0 & "  $"
        lblCashUsed.Text = 0 & "  $"
        lblError.Text = ""

        'Step 3: Execute the heuristic algorithm
        'Loop until the currentReturn is the required return
        Do While (currentReturn < MINRET)

            Dim delta As Double = 0.01
            Dim AR As Double = CDbl(dsGetStockWorth.Tables(0).Rows(bestStock).Item(1))
            Dim VAR As Double = CDbl(dsGetStockWorth.Tables(0).Rows(bestStock).Item(2))

            'When the room for an investment is less than the delta increment
            If (MAXWT - weights(bestStock)) < delta Then
                delta = MAXWT - weights(bestStock)
            End If

            weights(bestStock) += delta
            currentReturn += delta * AR
            worth(bestStock) = AR / (VAR + ((weights(bestStock) ^ 2) * VAR))

            'Find the bestStock after this delta investment
            maxWorth = 0
            For i = 0 To numStocks - 1
                If ((worth(i) > maxWorth) And (weights(i) < MAXWT)) Then
                    bestStock = i
                    maxWorth = worth(i)
                End If
            Next
            If (maxWorth = 0) Then
                lblError.Text = "Infeasible solution. Input is too constrained."
                Exit Sub
            End If

        Loop

        'Step 4: Set the final output of the optimization problem
        finalReturn = currentReturn
        For i = 0 To numStocks - 1
            cashUsed += weights(i) * CASH
            finalVariance += (weights(i) ^ 2) * _
            CDbl(dsGetStockWorth.Tables(0).Rows(i).Item(2))
        Next
        lblVar.Text = Math.Round(finalVariance, 2).ToString
        lblReturn.Text = "$ " & Math.Round((finalReturn * CASH), 2)
        lblCashUsed.Text = "$ " & cashUsed

        'Step 5: Show the suggested investment to the user
        'Insert the results into the tblOptimization table
        dsOptTable.DeleteCommand = "DELETE * from tblOptimization"
        dsOptTable.Delete()
        For i = 0 To numStocks - 1
            dsOptTable.InsertCommand = _
                "INSERT INTO tblOptimization (Symbol,Cash) VALUES ('" _
                & dsGetStockWorth.Tables(0).Rows(i).Item(0) & "'," _
                & Math.Round((weights(i) * CASH), 2) & ")"
            dsOptTable.Insert()
        Next
        'Show results on the DataGridView control
        dataGridResults.DataBind()
        'Show the pie-chart crystal report
        Dim myConnection As New OleDbConnection
        Dim myCommand As New OleDbCommand
        Dim myDA As New OleDbDataAdapter
        Dim myDS As New DataSet

        myConnection.ConnectionString = ConfigurationManager.AppSettings("connString")
        myCommand.Connection = myConnection
        myCommand.CommandText = "Select * from tblOptimization"
        myCommand.CommandType = CommandType.Text
        myDA.SelectCommand = myCommand
        myCommand.Connection.Open()
        myDA.Fill(myDS, "tblOptimization")
        Dim oRpt As New CrystalDecisions.CrystalReports.Engine.ReportDocument
        oRpt.FileName = "C:\webdss\casestudies\Portfolio\rptOptimization.rpt"
        oRpt.SetDataSource(myDS)
        rptViewer1.ReportSource = oRpt
        myCommand.Connection.Close()

    End Sub

#End Region

  End Class
